/****************************************************************************
** Meta object code from reading C++ file 'bar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/search/bar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Search3BarE_t {};
} // unnamed namespace

template <> constexpr inline auto Search::Bar::qt_create_metaobjectdata<qt_meta_tag_ZN6Search3BarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Search::Bar",
        "urlChangeRequested",
        "",
        "QUrl",
        "url",
        "focusViewRequest",
        "showMessage",
        "message",
        "KMessageWidget::MessageType",
        "messageType",
        "showInstallationProgress",
        "currentlyRunningTaskTitle",
        "installationProgressPercent",
        "visibilityChanged",
        "visible",
        "slotConfigurationChanged",
        "Search::DolphinQuery",
        "searchConfiguration",
        "slotSearchTermEdited",
        "text",
        "slotReturnPressed",
        "commitCurrentConfiguration",
        "slotSaveSearch"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlChangeRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'focusViewRequest'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(const QString &, KMessageWidget::MessageType)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'showInstallationProgress'
        QtMocHelpers::SignalData<void(const QString &, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'visibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotConfigurationChanged'
        QtMocHelpers::SlotData<void(const Search::DolphinQuery &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotSearchTermEdited'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotReturnPressed'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'commitCurrentConfiguration'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSaveSearch'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Bar, qt_meta_tag_ZN6Search3BarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Search::Bar::staticMetaObject = { {
    QMetaObject::SuperData::link<AnimatedHeightWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Search3BarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Search3BarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Search3BarE_t>.metaTypes,
    nullptr
} };

void Search::Bar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Bar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlChangeRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->focusViewRequest(); break;
        case 2: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KMessageWidget::MessageType>>(_a[2]))); break;
        case 3: _t->showInstallationProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->visibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotConfigurationChanged((*reinterpret_cast<std::add_pointer_t<Search::DolphinQuery>>(_a[1]))); break;
        case 6: _t->slotSearchTermEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->slotReturnPressed(); break;
        case 8: _t->commitCurrentConfiguration(); break;
        case 9: _t->slotSaveSearch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Bar::*)(const QUrl & )>(_a, &Bar::urlChangeRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Bar::*)()>(_a, &Bar::focusViewRequest, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Bar::*)(const QString & , KMessageWidget::MessageType )>(_a, &Bar::showMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Bar::*)(const QString & , int )>(_a, &Bar::showInstallationProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Bar::*)(bool )>(_a, &Bar::visibilityChanged, 4))
            return;
    }
}

const QMetaObject *Search::Bar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Search::Bar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Search3BarE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UpdatableStateInterface"))
        return static_cast< UpdatableStateInterface*>(this);
    return AnimatedHeightWidget::qt_metacast(_clname);
}

int Search::Bar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AnimatedHeightWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Search::Bar::urlChangeRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Search::Bar::focusViewRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Search::Bar::showMessage(const QString & _t1, KMessageWidget::MessageType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Search::Bar::showInstallationProgress(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Search::Bar::visibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
