/****************************************************************************
** Meta object code from reading C++ file 'filterbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/filterbar/filterbar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filterbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9FilterBarE_t {};
} // unnamed namespace

template <> constexpr inline auto FilterBar::qt_create_metaobjectdata<qt_meta_tag_ZN9FilterBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FilterBar",
        "filterChanged",
        "",
        "nameFilter",
        "caseSensitiveChanged",
        "caseSensitive",
        "filterModeChanged",
        "KFileItemModelFilter::FilterMode",
        "mode",
        "closeRequest",
        "focusViewRequest",
        "clear",
        "clearIfUnlocked",
        "slotToggleLockButton",
        "checked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'filterChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'caseSensitiveChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'filterModeChanged'
        QtMocHelpers::SignalData<void(KFileItemModelFilter::FilterMode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'closeRequest'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'focusViewRequest'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearIfUnlocked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleLockButton'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FilterBar, qt_meta_tag_ZN9FilterBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FilterBar::staticMetaObject = { {
    QMetaObject::SuperData::link<AnimatedHeightWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FilterBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FilterBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9FilterBarE_t>.metaTypes,
    nullptr
} };

void FilterBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FilterBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->filterChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->caseSensitiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->filterModeChanged((*reinterpret_cast<std::add_pointer_t<KFileItemModelFilter::FilterMode>>(_a[1]))); break;
        case 3: _t->closeRequest(); break;
        case 4: _t->focusViewRequest(); break;
        case 5: _t->clear(); break;
        case 6: _t->clearIfUnlocked(); break;
        case 7: _t->slotToggleLockButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FilterBar::*)(const QString & )>(_a, &FilterBar::filterChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FilterBar::*)(bool )>(_a, &FilterBar::caseSensitiveChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FilterBar::*)(KFileItemModelFilter::FilterMode )>(_a, &FilterBar::filterModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FilterBar::*)()>(_a, &FilterBar::closeRequest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FilterBar::*)()>(_a, &FilterBar::focusViewRequest, 4))
            return;
    }
}

const QMetaObject *FilterBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FilterBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FilterBarE_t>.strings))
        return static_cast<void*>(this);
    return AnimatedHeightWidget::qt_metacast(_clname);
}

int FilterBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AnimatedHeightWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void FilterBar::filterChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FilterBar::caseSensitiveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FilterBar::filterModeChanged(KFileItemModelFilter::FilterMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FilterBar::closeRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void FilterBar::focusViewRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
