/****************************************************************************
** Meta object code from reading C++ file 'kfileitemmodelrolesupdater.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/kitemviews/kfileitemmodelrolesupdater.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfileitemmodelrolesupdater.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t {};
} // unnamed namespace

template <> constexpr inline auto KFileItemModelRolesUpdater::qt_create_metaobjectdata<qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFileItemModelRolesUpdater",
        "previewJobFinished",
        "",
        "slotItemsInserted",
        "KItemRangeList",
        "itemRanges",
        "slotItemsRemoved",
        "slotItemsMoved",
        "KItemRange",
        "itemRange",
        "QList<int>",
        "movedToIndexes",
        "slotItemsChanged",
        "QSet<QByteArray>",
        "roles",
        "slotSortRoleChanged",
        "current",
        "previous",
        "slotGotPreview",
        "KFileItem",
        "item",
        "QPixmap",
        "pixmap",
        "slotPreviewFailed",
        "slotPreviewJobFinished",
        "slotHoverSequenceGotPreview",
        "slotHoverSequencePreviewFailed",
        "slotHoverSequencePreviewJobFinished",
        "slotOverlaysChanged",
        "QUrl",
        "url",
        "resolveNextSortRole",
        "resolveNextPendingRoles",
        "resolveRecentlyChangedItems",
        "applyChangedBalooRoles",
        "file",
        "applyChangedBalooRolesForItem",
        "slotDirectoryContentsCountReceived",
        "path",
        "count",
        "size"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'previewJobFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemsInserted'
        QtMocHelpers::SlotData<void(const KItemRangeList &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotItemsRemoved'
        QtMocHelpers::SlotData<void(const KItemRangeList &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotItemsMoved'
        QtMocHelpers::SlotData<void(KItemRange, const QList<int> &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotItemsChanged'
        QtMocHelpers::SlotData<void(const KItemRangeList &, const QSet<QByteArray> &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotSortRoleChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 16 }, { QMetaType::QByteArray, 17 },
        }}),
        // Slot 'slotGotPreview'
        QtMocHelpers::SlotData<void(const KFileItem &, const QPixmap &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotPreviewFailed'
        QtMocHelpers::SlotData<void(const KFileItem &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotPreviewJobFinished'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHoverSequenceGotPreview'
        QtMocHelpers::SlotData<void(const KFileItem &, const QPixmap &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Slot 'slotHoverSequencePreviewFailed'
        QtMocHelpers::SlotData<void(const KFileItem &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotHoverSequencePreviewJobFinished'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOverlaysChanged'
        QtMocHelpers::SlotData<void(const QUrl &, const QStringList &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QStringList, 2 },
        }}),
        // Slot 'resolveNextSortRole'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resolveNextPendingRoles'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resolveRecentlyChangedItems'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyChangedBalooRoles'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 },
        }}),
        // Slot 'applyChangedBalooRolesForItem'
        QtMocHelpers::SlotData<void(const KFileItem &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 35 },
        }}),
        // Slot 'slotDirectoryContentsCountReceived'
        QtMocHelpers::SlotData<void(const QString &, int, long long)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 38 }, { QMetaType::Int, 39 }, { QMetaType::LongLong, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KFileItemModelRolesUpdater, qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFileItemModelRolesUpdater::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>.metaTypes,
    nullptr
} };

void KFileItemModelRolesUpdater::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFileItemModelRolesUpdater *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->previewJobFinished(); break;
        case 1: _t->slotItemsInserted((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 2: _t->slotItemsRemoved((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 3: _t->slotItemsMoved((*reinterpret_cast<std::add_pointer_t<KItemRange>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 4: _t->slotItemsChanged((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[2]))); break;
        case 5: _t->slotSortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 6: _t->slotGotPreview((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 7: _t->slotPreviewFailed((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 8: _t->slotPreviewJobFinished(); break;
        case 9: _t->slotHoverSequenceGotPreview((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 10: _t->slotHoverSequencePreviewFailed((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 11: _t->slotHoverSequencePreviewJobFinished(); break;
        case 12: _t->slotOverlaysChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 13: _t->resolveNextSortRole(); break;
        case 14: _t->resolveNextPendingRoles(); break;
        case 15: _t->resolveRecentlyChangedItems(); break;
        case 16: _t->applyChangedBalooRoles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->applyChangedBalooRolesForItem((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 18: _t->slotDirectoryContentsCountReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KFileItemModelRolesUpdater::*)()>(_a, &KFileItemModelRolesUpdater::previewJobFinished, 0))
            return;
    }
}

const QMetaObject *KFileItemModelRolesUpdater::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFileItemModelRolesUpdater::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KFileItemModelRolesUpdaterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KFileItemModelRolesUpdater::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void KFileItemModelRolesUpdater::previewJobFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
