/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.DolphinMainWindow.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "dolphinmainwindowadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class MainWindowAdaptor
 */

MainWindowAdaptor::MainWindowAdaptor(DolphinMainWindow *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MainWindowAdaptor::~MainWindowAdaptor()
{
    // destructor
}

void MainWindowAdaptor::activateWindow(const QString &activationToken)
{
    // handle method call org.kde.dolphin.MainWindow.activateWindow
    parent()->activateWindow(activationToken);
}

bool MainWindowAdaptor::isActiveWindow()
{
    // handle method call org.kde.dolphin.MainWindow.isActiveWindow
    return parent()->isActiveWindow();
}

bool MainWindowAdaptor::isItemVisibleInAnyView(const QString &urlOfItem)
{
    // handle method call org.kde.dolphin.MainWindow.isItemVisibleInAnyView
    return parent()->isItemVisibleInAnyView(urlOfItem);
}

bool MainWindowAdaptor::isUrlOpen(const QString &url)
{
    // handle method call org.kde.dolphin.MainWindow.isUrlOpen
    return parent()->isUrlOpen(url);
}

void MainWindowAdaptor::openDirectories(const QStringList &dirs, bool splitView)
{
    // handle method call org.kde.dolphin.MainWindow.openDirectories
    parent()->openDirectories(dirs, splitView);
}

void MainWindowAdaptor::openFiles(const QStringList &files, bool splitView)
{
    // handle method call org.kde.dolphin.MainWindow.openFiles
    parent()->openFiles(files, splitView);
}

void MainWindowAdaptor::pasteIntoFolder()
{
    // handle method call org.kde.dolphin.MainWindow.pasteIntoFolder
    parent()->pasteIntoFolder();
}

void MainWindowAdaptor::quit()
{
    // handle method call org.kde.dolphin.MainWindow.quit
    parent()->quit();
}

void MainWindowAdaptor::slotSplitViewChanged()
{
    // handle method call org.kde.dolphin.MainWindow.slotSplitViewChanged
    parent()->slotSplitViewChanged();
}

void MainWindowAdaptor::slotTabBarChanged()
{
    // handle method call org.kde.dolphin.MainWindow.slotTabBarChanged
    parent()->slotTabBarChanged();
}


#include "moc_dolphinmainwindowadaptor.cpp"
