/****************************************************************************
** Meta object code from reading C++ file 'dolphintabpage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/dolphintabpage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphintabpage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DolphinTabPageE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinTabPage::qt_create_metaobjectdata<qt_meta_tag_ZN14DolphinTabPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinTabPage",
        "activeViewChanged",
        "",
        "DolphinViewContainer*",
        "viewContainer",
        "activeViewUrlChanged",
        "QUrl",
        "url",
        "splitterMoved",
        "pos",
        "slotAnimationFinished",
        "slotAnimationValueChanged",
        "QVariant",
        "value",
        "slotViewActivated",
        "slotViewUrlRedirection",
        "oldUrl",
        "newUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeViewChanged'
        QtMocHelpers::SignalData<void(DolphinViewContainer *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'activeViewUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'splitterMoved'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'slotAnimationFinished'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAnimationValueChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotViewActivated'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotViewUrlRedirection'
        QtMocHelpers::SlotData<void(const QUrl &, const QUrl &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 16 }, { 0x80000000 | 6, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinTabPage, qt_meta_tag_ZN14DolphinTabPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinTabPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DolphinTabPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DolphinTabPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DolphinTabPageE_t>.metaTypes,
    nullptr
} };

void DolphinTabPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinTabPage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeViewChanged((*reinterpret_cast<std::add_pointer_t<DolphinViewContainer*>>(_a[1]))); break;
        case 1: _t->activeViewUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->splitterMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->slotAnimationFinished(); break;
        case 4: _t->slotAnimationValueChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 5: _t->slotViewActivated(); break;
        case 6: _t->slotViewUrlRedirection((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DolphinTabPage::*)(DolphinViewContainer * )>(_a, &DolphinTabPage::activeViewChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinTabPage::*)(const QUrl & )>(_a, &DolphinTabPage::activeViewUrlChanged, 1))
            return;
    }
}

const QMetaObject *DolphinTabPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinTabPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DolphinTabPageE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DolphinTabPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void DolphinTabPage::activeViewChanged(DolphinViewContainer * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DolphinTabPage::activeViewUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinTabPageSplitterHandle::qt_create_metaobjectdata<qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinTabPageSplitterHandle"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinTabPageSplitterHandle, qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinTabPageSplitterHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitterHandle::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>.metaTypes,
    nullptr
} };

void DolphinTabPageSplitterHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinTabPageSplitterHandle *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DolphinTabPageSplitterHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinTabPageSplitterHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DolphinTabPageSplitterHandleE_t>.strings))
        return static_cast<void*>(this);
    return QSplitterHandle::qt_metacast(_clname);
}

int DolphinTabPageSplitterHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitterHandle::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN22DolphinTabPageSplitterE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinTabPageSplitter::qt_create_metaobjectdata<qt_meta_tag_ZN22DolphinTabPageSplitterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinTabPageSplitter"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinTabPageSplitter, qt_meta_tag_ZN22DolphinTabPageSplitterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinTabPageSplitter::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DolphinTabPageSplitterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DolphinTabPageSplitterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22DolphinTabPageSplitterE_t>.metaTypes,
    nullptr
} };

void DolphinTabPageSplitter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinTabPageSplitter *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DolphinTabPageSplitter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinTabPageSplitter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DolphinTabPageSplitterE_t>.strings))
        return static_cast<void*>(this);
    return QSplitter::qt_metacast(_clname);
}

int DolphinTabPageSplitter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
