/****************************************************************************
** Meta object code from reading C++ file 'dolphinmainwindowinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "dolphinmainwindowinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphinmainwindowinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeDolphinMainWindowInterface::qt_create_metaobjectdata<qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeDolphinMainWindowInterface",
        "settingsChanged",
        "",
        "activateWindow",
        "QDBusPendingReply<>",
        "activationToken",
        "isActiveWindow",
        "QDBusPendingReply<bool>",
        "isItemVisibleInAnyView",
        "urlOfItem",
        "isUrlOpen",
        "url",
        "openDirectories",
        "dirs",
        "splitView",
        "openFiles",
        "files",
        "pasteIntoFolder",
        "quit",
        "slotSplitViewChanged",
        "slotTabBarChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateWindow'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(3, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'isActiveWindow'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'isItemVisibleInAnyView'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(8, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'isUrlOpen'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(10, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'openDirectories'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QStringList &, bool)>(12, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QStringList, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'openFiles'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QStringList &, bool)>(15, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QStringList, 16 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'pasteIntoFolder'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(17, 2, QMC::AccessPublic, 0x80000000 | 4),
        // Slot 'quit'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(18, 2, QMC::AccessPublic, 0x80000000 | 4),
        // Slot 'slotSplitViewChanged'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(19, 2, QMC::AccessPublic, 0x80000000 | 4),
        // Slot 'slotTabBarChanged'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(20, 2, QMC::AccessPublic, 0x80000000 | 4),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeDolphinMainWindowInterface, qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeDolphinMainWindowInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeDolphinMainWindowInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeDolphinMainWindowInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->settingsChanged(); break;
        case 1: { QDBusPendingReply<> _r = _t->activateWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<bool> _r = _t->isActiveWindow();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<bool> _r = _t->isItemVisibleInAnyView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<bool> _r = _t->isUrlOpen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->openDirectories((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->openFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->pasteIntoFolder();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->quit();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->slotSplitViewChanged();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->slotTabBarChanged();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeDolphinMainWindowInterface::*)()>(_a, &OrgKdeDolphinMainWindowInterface::settingsChanged, 0))
            return;
    }
}

const QMetaObject *OrgKdeDolphinMainWindowInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeDolphinMainWindowInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32OrgKdeDolphinMainWindowInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeDolphinMainWindowInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeDolphinMainWindowInterface::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
