/****************************************************************************
** Meta object code from reading C++ file 'dolphinmainwindowadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "dolphinmainwindowadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphinmainwindowadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17MainWindowAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindowAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN17MainWindowAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindowAdaptor",
        "D-Bus Interface",
        "org.kde.dolphin.MainWindow",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.dolphin.MainWindow\">\n    <signal nam"
        "e=\"settingsChanged\"/>\n    <method name=\"openDirectories\">\n  "
        "    <arg direction=\"in\" type=\"as\" name=\"dirs\"/>\n      <arg "
        "direction=\"in\" type=\"b\" name=\"splitView\"/>\n    </method>\n "
        "   <method name=\"openFiles\">\n      <arg direction=\"in\" type=\""
        "as\" name=\"files\"/>\n      <arg direction=\"in\" type=\"b\" name"
        "=\"splitView\"/>\n    </method>\n    <method name=\"activateWindow"
        "\">\n      <arg direction=\"in\" type=\"s\" name=\"activationToken"
        "\"/>\n    </method>\n    <method name=\"isActiveWindow\">\n      <"
        "arg direction=\"out\" type=\"b\"/>\n    </method>\n    <method nam"
        "e=\"isUrlOpen\">\n      <arg direction=\"out\" type=\"b\"/>\n     "
        " <arg direction=\"in\" type=\"s\" name=\"url\"/>\n    </method>\n "
        "   <method name=\"isItemVisibleInAnyView\">\n      <arg direction="
        "\"out\" type=\"b\"/>\n      <arg direction=\"in\" type=\"s\" name="
        "\"urlOfItem\"/>\n    </method>\n    <method name=\"pasteIntoFolder"
        "\"/>\n    <method name=\"quit\"/>\n    <method name=\"slotSplitVie"
        "wChanged\"/>\n    <method name=\"slotTabBarChanged\"/>\n  </interf"
        "ace>\n",
        "settingsChanged",
        "",
        "activateWindow",
        "activationToken",
        "isActiveWindow",
        "isItemVisibleInAnyView",
        "urlOfItem",
        "isUrlOpen",
        "url",
        "openDirectories",
        "dirs",
        "splitView",
        "openFiles",
        "files",
        "pasteIntoFolder",
        "quit",
        "slotSplitViewChanged",
        "slotTabBarChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateWindow'
        QtMocHelpers::SlotData<void(const QString &)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'isActiveWindow'
        QtMocHelpers::SlotData<bool()>(9, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isItemVisibleInAnyView'
        QtMocHelpers::SlotData<bool(const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'isUrlOpen'
        QtMocHelpers::SlotData<bool(const QString &)>(12, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'openDirectories'
        QtMocHelpers::SlotData<void(const QStringList &, bool)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'openFiles'
        QtMocHelpers::SlotData<void(const QStringList &, bool)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 18 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'pasteIntoFolder'
        QtMocHelpers::SlotData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSplitViewChanged'
        QtMocHelpers::SlotData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTabBarChanged'
        QtMocHelpers::SlotData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<MainWindowAdaptor, qt_meta_tag_ZN17MainWindowAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MainWindowAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MainWindowAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MainWindowAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17MainWindowAdaptorE_t>.metaTypes,
    nullptr
} };

void MainWindowAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindowAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->settingsChanged(); break;
        case 1: _t->activateWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { bool _r = _t->isActiveWindow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->isItemVisibleInAnyView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->isUrlOpen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->openDirectories((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->openFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->pasteIntoFolder(); break;
        case 8: _t->quit(); break;
        case 9: _t->slotSplitViewChanged(); break;
        case 10: _t->slotTabBarChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindowAdaptor::*)()>(_a, &MainWindowAdaptor::settingsChanged, 0))
            return;
    }
}

const QMetaObject *MainWindowAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindowAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MainWindowAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int MainWindowAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void MainWindowAdaptor::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
