/****************************************************************************
** Meta object code from reading C++ file 'kitemlistcontainer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/kitemviews/kitemlistcontainer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kitemlistcontainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18KItemListContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto KItemListContainer::qt_create_metaobjectdata<qt_meta_tag_ZN18KItemListContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItemListContainer",
        "slotScrollOrientationChanged",
        "",
        "Qt::Orientation",
        "current",
        "previous",
        "slotModelChanged",
        "KItemModelBase*",
        "slotViewChanged",
        "KItemListView*",
        "scrollTo",
        "offset",
        "updateScrollOffsetScrollBar",
        "updateItemOffsetScrollBar",
        "stopScroller"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotScrollOrientationChanged'
        QtMocHelpers::SlotData<void(Qt::Orientation, Qt::Orientation)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'slotModelChanged'
        QtMocHelpers::SlotData<void(KItemModelBase *, KItemModelBase *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 4 }, { 0x80000000 | 7, 5 },
        }}),
        // Slot 'slotViewChanged'
        QtMocHelpers::SlotData<void(KItemListView *, KItemListView *)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 4 }, { 0x80000000 | 9, 5 },
        }}),
        // Slot 'scrollTo'
        QtMocHelpers::SlotData<void(qreal)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Slot 'updateScrollOffsetScrollBar'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateItemOffsetScrollBar'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'stopScroller'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KItemListContainer, qt_meta_tag_ZN18KItemListContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItemListContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractScrollArea::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KItemListContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KItemListContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18KItemListContainerE_t>.metaTypes,
    nullptr
} };

void KItemListContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KItemListContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotScrollOrientationChanged((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[2]))); break;
        case 1: _t->slotModelChanged((*reinterpret_cast<std::add_pointer_t<KItemModelBase*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemModelBase*>>(_a[2]))); break;
        case 2: _t->slotViewChanged((*reinterpret_cast<std::add_pointer_t<KItemListView*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemListView*>>(_a[2]))); break;
        case 3: _t->scrollTo((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 4: _t->updateScrollOffsetScrollBar(); break;
        case 5: _t->updateItemOffsetScrollBar(); break;
        case 6: _t->stopScroller(); break;
        default: ;
        }
    }
}

const QMetaObject *KItemListContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItemListContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KItemListContainerE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractScrollArea::qt_metacast(_clname);
}

int KItemListContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
