/****************************************************************************
** Meta object code from reading C++ file 'kitemlistview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/kitemviews/kitemlistview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kitemlistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KItemListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KItemListView::qt_create_metaobjectdata<qt_meta_tag_ZN13KItemListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KItemListView",
        "scrollOrientationChanged",
        "",
        "Qt::Orientation",
        "current",
        "previous",
        "scrollOffsetChanged",
        "maximumScrollOffsetChanged",
        "itemOffsetChanged",
        "maximumItemOffsetChanged",
        "scrollTo",
        "newOffset",
        "sortOrderChanged",
        "Qt::SortOrder",
        "sortRoleChanged",
        "visibleRolesChanged",
        "QList<QByteArray>",
        "roleEditingCanceled",
        "index",
        "role",
        "QVariant",
        "value",
        "roleEditingFinished",
        "scrollingStopped",
        "columnHovered",
        "roleIndex",
        "columnUnHovered",
        "slotItemsInserted",
        "KItemRangeList",
        "itemRanges",
        "slotItemsRemoved",
        "slotItemsMoved",
        "KItemRange",
        "itemRange",
        "QList<int>",
        "movedToIndexes",
        "slotItemsChanged",
        "QSet<QByteArray>",
        "roles",
        "slotGroupsChanged",
        "slotGroupedSortingChanged",
        "slotSortOrderChanged",
        "slotSortRoleChanged",
        "slotGroupRoleChanged",
        "slotCurrentChanged",
        "slotSelectionChanged",
        "KItemSet",
        "slotAnimationFinished",
        "QGraphicsWidget*",
        "widget",
        "KItemListViewAnimation::AnimationType",
        "type",
        "slotAnimationStarted",
        "endValue",
        "slotRubberBandPosChanged",
        "slotRubberBandActivationChanged",
        "active",
        "slotHeaderColumnWidthChanged",
        "currentWidth",
        "previousWidth",
        "slotSidePaddingChanged",
        "width",
        "slotHeaderColumnMoved",
        "currentIndex",
        "previousIndex",
        "triggerAutoScrolling",
        "slotGeometryOfGroupHeaderParentChanged",
        "slotRoleEditingCanceled",
        "slotRoleEditingFinished",
        "scrollOffset",
        "itemOffset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scrollOrientationChanged'
        QtMocHelpers::SignalData<void(Qt::Orientation, Qt::Orientation)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Signal 'scrollOffsetChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'maximumScrollOffsetChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'itemOffsetChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'maximumItemOffsetChanged'
        QtMocHelpers::SignalData<void(qreal, qreal)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'scrollTo'
        QtMocHelpers::SignalData<void(qreal)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Signal 'sortOrderChanged'
        QtMocHelpers::SignalData<void(Qt::SortOrder, Qt::SortOrder)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 4 }, { 0x80000000 | 13, 5 },
        }}),
        // Signal 'sortRoleChanged'
        QtMocHelpers::SignalData<void(const QByteArray &, const QByteArray &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Signal 'visibleRolesChanged'
        QtMocHelpers::SignalData<void(const QList<QByteArray> &, const QList<QByteArray> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 4 }, { 0x80000000 | 16, 5 },
        }}),
        // Signal 'roleEditingCanceled'
        QtMocHelpers::SignalData<void(int, const QByteArray &, const QVariant &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QByteArray, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'roleEditingFinished'
        QtMocHelpers::SignalData<void(int, const QByteArray &, const QVariant &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QByteArray, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'scrollingStopped'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnHovered'
        QtMocHelpers::SignalData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Signal 'columnUnHovered'
        QtMocHelpers::SignalData<void(int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'slotItemsInserted'
        QtMocHelpers::SlotData<void(const KItemRangeList &)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotItemsRemoved'
        QtMocHelpers::SlotData<void(const KItemRangeList &)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotItemsMoved'
        QtMocHelpers::SlotData<void(const KItemRange &, const QList<int> &)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { 0x80000000 | 34, 35 },
        }}),
        // Slot 'slotItemsChanged'
        QtMocHelpers::SlotData<void(const KItemRangeList &, const QSet<QByteArray> &)>(36, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 28, 29 }, { 0x80000000 | 37, 38 },
        }}),
        // Slot 'slotGroupsChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotGroupedSortingChanged'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'slotSortOrderChanged'
        QtMocHelpers::SlotData<void(Qt::SortOrder, Qt::SortOrder)>(41, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 4 }, { 0x80000000 | 13, 5 },
        }}),
        // Slot 'slotSortRoleChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'slotGroupRoleChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, const QByteArray &)>(43, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(int, int)>(44, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const KItemSet &, const KItemSet &)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 46, 4 }, { 0x80000000 | 46, 5 },
        }}),
        // Slot 'slotAnimationFinished'
        QtMocHelpers::SlotData<void(QGraphicsWidget *, KItemListViewAnimation::AnimationType)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { 0x80000000 | 50, 51 },
        }}),
        // Slot 'slotAnimationStarted'
        QtMocHelpers::SlotData<void(QGraphicsWidget *, KItemListViewAnimation::AnimationType, const QVariant &)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { 0x80000000 | 50, 51 }, { 0x80000000 | 20, 53 },
        }}),
        // Slot 'slotRubberBandPosChanged'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRubberBandActivationChanged'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'slotHeaderColumnWidthChanged'
        QtMocHelpers::SlotData<void(const QByteArray &, qreal, qreal)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 19 }, { QMetaType::QReal, 58 }, { QMetaType::QReal, 59 },
        }}),
        // Slot 'slotSidePaddingChanged'
        QtMocHelpers::SlotData<void(qreal)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 61 },
        }}),
        // Slot 'slotHeaderColumnMoved'
        QtMocHelpers::SlotData<void(const QByteArray &, int, int)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 19 }, { QMetaType::Int, 63 }, { QMetaType::Int, 64 },
        }}),
        // Slot 'triggerAutoScrolling'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGeometryOfGroupHeaderParentChanged'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRoleEditingCanceled'
        QtMocHelpers::SlotData<void(int, const QByteArray &, const QVariant &)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QByteArray, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotRoleEditingFinished'
        QtMocHelpers::SlotData<void(int, const QByteArray &, const QVariant &)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::QByteArray, 19 }, { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'scrollOffset'
        QtMocHelpers::PropertyData<qreal>(69, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'itemOffset'
        QtMocHelpers::PropertyData<qreal>(70, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KItemListView, qt_meta_tag_ZN13KItemListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KItemListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KItemListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KItemListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KItemListViewE_t>.metaTypes,
    nullptr
} };

void KItemListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KItemListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scrollOrientationChanged((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[2]))); break;
        case 1: _t->scrollOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 2: _t->maximumScrollOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 3: _t->itemOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 4: _t->maximumItemOffsetChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 5: _t->scrollTo((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->sortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 7: _t->sortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 8: _t->visibleRolesChanged((*reinterpret_cast<std::add_pointer_t<QList<QByteArray>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QByteArray>>>(_a[2]))); break;
        case 9: _t->roleEditingCanceled((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 10: _t->roleEditingFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 11: _t->scrollingStopped(); break;
        case 12: _t->columnHovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->columnUnHovered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->slotItemsInserted((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 15: _t->slotItemsRemoved((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1]))); break;
        case 16: _t->slotItemsMoved((*reinterpret_cast<std::add_pointer_t<KItemRange>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 17: _t->slotItemsChanged((*reinterpret_cast<std::add_pointer_t<KItemRangeList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[2]))); break;
        case 18: _t->slotGroupsChanged(); break;
        case 19: _t->slotGroupedSortingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->slotSortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 21: _t->slotSortRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 22: _t->slotGroupRoleChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 23: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 24: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<KItemSet>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemSet>>(_a[2]))); break;
        case 25: _t->slotAnimationFinished((*reinterpret_cast<std::add_pointer_t<QGraphicsWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemListViewAnimation::AnimationType>>(_a[2]))); break;
        case 26: _t->slotAnimationStarted((*reinterpret_cast<std::add_pointer_t<QGraphicsWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KItemListViewAnimation::AnimationType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 27: _t->slotRubberBandPosChanged(); break;
        case 28: _t->slotRubberBandActivationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->slotHeaderColumnWidthChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 30: _t->slotSidePaddingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 31: _t->slotHeaderColumnMoved((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 32: _t->triggerAutoScrolling(); break;
        case 33: _t->slotGeometryOfGroupHeaderParentChanged(); break;
        case 34: _t->slotRoleEditingCanceled((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 35: _t->slotRoleEditingFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QByteArray> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsWidget* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGraphicsWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(Qt::Orientation , Qt::Orientation )>(_a, &KItemListView::scrollOrientationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(qreal , qreal )>(_a, &KItemListView::scrollOffsetChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(qreal , qreal )>(_a, &KItemListView::maximumScrollOffsetChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(qreal , qreal )>(_a, &KItemListView::itemOffsetChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(qreal , qreal )>(_a, &KItemListView::maximumItemOffsetChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(qreal )>(_a, &KItemListView::scrollTo, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(Qt::SortOrder , Qt::SortOrder )>(_a, &KItemListView::sortOrderChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(const QByteArray & , const QByteArray & )>(_a, &KItemListView::sortRoleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(const QList<QByteArray> & , const QList<QByteArray> & )>(_a, &KItemListView::visibleRolesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(int , const QByteArray & , const QVariant & )>(_a, &KItemListView::roleEditingCanceled, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(int , const QByteArray & , const QVariant & )>(_a, &KItemListView::roleEditingFinished, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)()>(_a, &KItemListView::scrollingStopped, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(int )>(_a, &KItemListView::columnHovered, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KItemListView::*)(int )>(_a, &KItemListView::columnUnHovered, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->scrollOffset(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->itemOffset(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScrollOffset(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setItemOffset(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KItemListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KItemListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KItemListViewE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsWidget::qt_metacast(_clname);
}

int KItemListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KItemListView::scrollOrientationChanged(Qt::Orientation _t1, Qt::Orientation _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KItemListView::scrollOffsetChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KItemListView::maximumScrollOffsetChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KItemListView::itemOffsetChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KItemListView::maximumItemOffsetChanged(qreal _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KItemListView::scrollTo(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KItemListView::sortOrderChanged(Qt::SortOrder _t1, Qt::SortOrder _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void KItemListView::sortRoleChanged(const QByteArray & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void KItemListView::visibleRolesChanged(const QList<QByteArray> & _t1, const QList<QByteArray> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void KItemListView::roleEditingCanceled(int _t1, const QByteArray & _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void KItemListView::roleEditingFinished(int _t1, const QByteArray & _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3);
}

// SIGNAL 11
void KItemListView::scrollingStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KItemListView::columnHovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void KItemListView::columnUnHovered(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
