// This file is generated by kconfig_compiler_kf6 from dolphin_contextmenusettings.kcfg.
// All changes you do to this file will be lost.

#include "dolphin_contextmenusettings.h"

#include <qglobal.h>
#include <QFile>

class ContextMenuSettingsHelper
{
  public:
    ContextMenuSettingsHelper() noexcept : q(nullptr) {}
    ~ContextMenuSettingsHelper() noexcept { delete q; q = nullptr; }
    ContextMenuSettingsHelper(const ContextMenuSettingsHelper&) = delete;
    ContextMenuSettingsHelper& operator=(const ContextMenuSettingsHelper&) = delete;
    ContextMenuSettings *q;
};
Q_GLOBAL_STATIC(ContextMenuSettingsHelper, s_globalContextMenuSettings)
ContextMenuSettings *ContextMenuSettings::self()
{
  if (!s_globalContextMenuSettings()->q) {
    new ContextMenuSettings;
    s_globalContextMenuSettings()->q->read();
  }

  return s_globalContextMenuSettings()->q;
}

ContextMenuSettings::ContextMenuSettings( )
  : KConfigSkeleton( QStringLiteral( "dolphinrc" ) )
{
  Q_ASSERT(!s_globalContextMenuSettings()->q);
  s_globalContextMenuSettings()->q = this;
  setCurrentGroup( QStringLiteral( "ContextMenu" ) );

  KConfigSkeleton::ItemBool  *itemShowCopyMoveMenu;
  itemShowCopyMoveMenu = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowCopyMoveMenu" ), mShowCopyMoveMenu, false );
  addItem( itemShowCopyMoveMenu, QStringLiteral( "ShowCopyMoveMenu" ) );
  KConfigSkeleton::ItemBool  *itemShowAddToPlaces;
  itemShowAddToPlaces = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowAddToPlaces" ), mShowAddToPlaces, true );
  addItem( itemShowAddToPlaces, QStringLiteral( "ShowAddToPlaces" ) );
  KConfigSkeleton::ItemBool  *itemShowSortBy;
  itemShowSortBy = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowSortBy" ), mShowSortBy, true );
  addItem( itemShowSortBy, QStringLiteral( "ShowSortBy" ) );
  KConfigSkeleton::ItemBool  *itemShowViewMode;
  itemShowViewMode = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowViewMode" ), mShowViewMode, true );
  addItem( itemShowViewMode, QStringLiteral( "ShowViewMode" ) );
  KConfigSkeleton::ItemBool  *itemShowOpenInNewTab;
  itemShowOpenInNewTab = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowOpenInNewTab" ), mShowOpenInNewTab, true );
  addItem( itemShowOpenInNewTab, QStringLiteral( "ShowOpenInNewTab" ) );
  KConfigSkeleton::ItemBool  *itemShowOpenInNewWindow;
  itemShowOpenInNewWindow = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowOpenInNewWindow" ), mShowOpenInNewWindow, true );
  addItem( itemShowOpenInNewWindow, QStringLiteral( "ShowOpenInNewWindow" ) );
  KConfigSkeleton::ItemBool  *itemShowOpenInSplitView;
  itemShowOpenInSplitView = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowOpenInSplitView" ), mShowOpenInSplitView, true );
  addItem( itemShowOpenInSplitView, QStringLiteral( "ShowOpenInSplitView" ) );
  KConfigSkeleton::ItemBool  *itemShowCopyLocation;
  itemShowCopyLocation = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowCopyLocation" ), mShowCopyLocation, true );
  addItem( itemShowCopyLocation, QStringLiteral( "ShowCopyLocation" ) );
  KConfigSkeleton::ItemBool  *itemShowDuplicateHere;
  itemShowDuplicateHere = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowDuplicateHere" ), mShowDuplicateHere, true );
  addItem( itemShowDuplicateHere, QStringLiteral( "ShowDuplicateHere" ) );
  KConfigSkeleton::ItemBool  *itemShowOpenTerminal;
  itemShowOpenTerminal = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowOpenTerminal" ), mShowOpenTerminal, true );
  addItem( itemShowOpenTerminal, QStringLiteral( "ShowOpenTerminal" ) );
  KConfigSkeleton::ItemBool  *itemShowCopyToOtherSplitView;
  itemShowCopyToOtherSplitView = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowCopyToOtherSplitView" ), mShowCopyToOtherSplitView, true );
  addItem( itemShowCopyToOtherSplitView, QStringLiteral( "ShowCopyToOtherSplitView" ) );
  KConfigSkeleton::ItemBool  *itemShowMoveToOtherSplitView;
  itemShowMoveToOtherSplitView = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowMoveToOtherSplitView" ), mShowMoveToOtherSplitView, true );
  addItem( itemShowMoveToOtherSplitView, QStringLiteral( "ShowMoveToOtherSplitView" ) );
}

ContextMenuSettings::~ContextMenuSettings()
{
  if (s_globalContextMenuSettings.exists() && !s_globalContextMenuSettings.isDestroyed()) {
    s_globalContextMenuSettings()->q = nullptr;
  }
}

