/****************************************************************************
** Meta object code from reading C++ file 'dolphinpart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-26.08.0/src/dolphinpart.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dolphinpart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11DolphinPartE_t {};
} // unnamed namespace

template <> constexpr inline auto DolphinPart::qt_create_metaobjectdata<qt_meta_tag_ZN11DolphinPartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DolphinPart",
        "viewModeChanged",
        "",
        "aboutToOpenURL",
        "slotMessage",
        "msg",
        "slotErrorMessage",
        "slotRequestItemInfo",
        "KFileItem",
        "item",
        "slotItemActivated",
        "slotItemsActivated",
        "KFileItemList",
        "items",
        "createNewWindow",
        "QUrl",
        "url",
        "slotOpenContextMenu",
        "QPoint",
        "pos",
        "_item",
        "selectedItems",
        "slotDirectoryRedirection",
        "oldUrl",
        "newUrl",
        "slotSelectionChanged",
        "selection",
        "updatePasteAction",
        "slotGoTriggered",
        "QAction*",
        "action",
        "slotEditMimeType",
        "slotSelectItemsMatchingPattern",
        "slotUnselectItemsMatchingPattern",
        "slotOpenTerminal",
        "slotFindFile",
        "updateNewMenu",
        "updateStatusBar",
        "updateProgress",
        "percent",
        "setNewFileMenuWorkingDirectory",
        "directory",
        "createDirectory",
        "setFilesToSelect",
        "QList<QUrl>",
        "files",
        "eventFilter",
        "QEvent*",
        "supportsUndo",
        "currentViewMode",
        "nameFilter",
        "filesToSelect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'viewModeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToOpenURL'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMessage'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotErrorMessage'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotRequestItemInfo'
        QtMocHelpers::SlotData<void(const KFileItem &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotItemActivated'
        QtMocHelpers::SlotData<void(const KFileItem &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotItemsActivated'
        QtMocHelpers::SlotData<void(const KFileItemList &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'createNewWindow'
        QtMocHelpers::SlotData<void(const QUrl &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotOpenContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &, const KFileItem &, const KFileItemList &, const QUrl &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 8, 20 }, { 0x80000000 | 12, 21 }, { 0x80000000 | 15, 2 },
        }}),
        // Slot 'slotDirectoryRedirection'
        QtMocHelpers::SlotData<void(const QUrl &, const QUrl &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 23 }, { 0x80000000 | 15, 24 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const KFileItemList &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 26 },
        }}),
        // Slot 'updatePasteAction'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGoTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotEditMimeType'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSelectItemsMatchingPattern'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUnselectItemsMatchingPattern'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenTerminal'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFindFile'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateNewMenu'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateStatusBar'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateProgress'
        QtMocHelpers::SlotData<void(int)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'setNewFileMenuWorkingDirectory'
        QtMocHelpers::SlotData<void(const QUrl &)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 41 },
        }}),
        // Slot 'createDirectory'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFilesToSelect'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'eventFilter'
        QtMocHelpers::SlotData<bool(QObject *, QEvent *)>(46, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 2 }, { 0x80000000 | 47, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'supportsUndo'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'currentViewMode'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'nameFilter'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'filesToSelect'
        QtMocHelpers::PropertyData<QList<QUrl>>(51, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DolphinPart, qt_meta_tag_ZN11DolphinPartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DolphinPart::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ReadOnlyPart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DolphinPartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DolphinPartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11DolphinPartE_t>.metaTypes,
    nullptr
} };

void DolphinPart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DolphinPart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->viewModeChanged(); break;
        case 1: _t->aboutToOpenURL(); break;
        case 2: _t->slotMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->slotErrorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->slotRequestItemInfo((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 5: _t->slotItemActivated((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 6: _t->slotItemsActivated((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 7: _t->createNewWindow((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->slotOpenContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[4]))); break;
        case 9: _t->slotDirectoryRedirection((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 10: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 11: _t->updatePasteAction(); break;
        case 12: _t->slotGoTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 13: _t->slotEditMimeType(); break;
        case 14: _t->slotSelectItemsMatchingPattern(); break;
        case 15: _t->slotUnselectItemsMatchingPattern(); break;
        case 16: _t->slotOpenTerminal(); break;
        case 17: _t->slotFindFile(); break;
        case 18: _t->updateNewMenu(); break;
        case 19: _t->updateStatusBar(); break;
        case 20: _t->updateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->setNewFileMenuWorkingDirectory((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 22: _t->createDirectory(); break;
        case 23: _t->setFilesToSelect((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 24: { bool _r = _t->eventFilter((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DolphinPart::*)()>(_a, &DolphinPart::viewModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DolphinPart::*)()>(_a, &DolphinPart::aboutToOpenURL, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->supportsUndo(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->currentViewMode(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->nameFilter(); break;
        case 3: *reinterpret_cast<QList<QUrl>*>(_v) = _t->filesToSelect(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCurrentViewMode(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setNameFilter(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setFilesToSelect(*reinterpret_cast<QList<QUrl>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DolphinPart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DolphinPart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DolphinPartE_t>.strings))
        return static_cast<void*>(this);
    return KParts::ReadOnlyPart::qt_metacast(_clname);
}

int DolphinPart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadOnlyPart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void DolphinPart::viewModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DolphinPart::aboutToOpenURL()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
